/*
 * Copyright (C) 2011 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by Alex Launi <alex.launi@canonical.com>
 *
 */

using Gee;
using Sqlite;

namespace Unity.MusicLens {

	public errordomain DatabaseError {
		FAILED_TO_OPEN
	}

	public enum Columns {
			TRACK_TITLE,
			TRACK_URI,
			TRACK_MIMETYPE,
			ALBUM_TITLE,
			ARTIST_NAME,
	}
	
	/**
	 * Abstracts talking to the banshee collection database
	 */
	public class BansheeCollection : GLib.Object
	{
		

		private const int MAX_RESULTS = 100;

		private Database db;

		public BansheeCollection () throws DatabaseError
		{
			int rc = Database.open ("/home/alex/.config/banshee-1/banshee.db", out db);
			
			if (rc != Sqlite.OK) {
				printerr ("failed to open db, %d, %s\n", rc, db.errmsg ());
				throw new DatabaseError.FAILED_TO_OPEN ("Failed to open banshee database");
			}	
		}

		/**
		 * Performs a search on the banshee db
		 */
		public void search (LensSearch? search, Dee.Model results_model, int max_results = -1)
		{
			int rc = 0;
			Statement stmt;
			// use a tree set to ensure we don't duplicate albums
			TreeSet<Album> albums = new TreeSet<Album> ();

			// BUILD SQL STATEMENT
			string sql = """SELECT CoreTracks.Title, CoreTracks.Uri, CoreTracks.MimeType, CoreAlbums.Title, CoreArtists.Name 
						  	FROM CoreTracks 
						  	CROSS JOIN CoreArtists, CoreAlbums 
						  	WHERE CoreArtists.ArtistID = CoreTracks.ArtistID AND CoreAlbums.AlbumID = CoreTracks.AlbumID 
						  								   AND CoreTracks.PrimarySourceID =1 
						  								   AND ((CoreArtists.NameLowered LIKE '%%%s%%' ESCAPE '\' 
						  								   	AND CoreArtists.NameLowered IS NOT NULL) 
						  								   OR (CoreAlbums.TitleLowered LIKE '%%%s%%' ESCAPE '\' 
						  								   	AND CoreAlbums.TitleLowered IS NOT NULL) 
						  								   OR (CoreTracks.TitleLowered LIKE '%%%s%%' ESCAPE '\' 
						  								   	AND CoreTracks.TitleLowered IS NOT NULL)) 
						  	LIMIT 0, %d;""".printf (search.search_string,
							  						search.search_string,
							  						search.search_string,
							  						max_results == -1 ? MAX_RESULTS : max_results);
			
			debug ("preparing to execute sql %s\n", sql);

			if ((rc = db.prepare_v2 (sql, -1, out stmt, null)) == 1)
				{
					// FIXME return a null result and print an error
					return;
				}


			do {
				rc = stmt.step ();
				switch (rc) {
				case Sqlite.DONE:
					break;
				case Sqlite.ROW:
					// translate the raw sql row into something we can use
					Track track = new Track ();
					track.Title = stmt.column_text (Columns.TRACK_TITLE);
					track.Artist = stmt.column_text (Columns.ARTIST_NAME);
					track.Uri = stmt.column_text (Columns.TRACK_URI);
					track.MimeType = stmt.column_text (Columns.TRACK_MIMETYPE);

					Album album = new Album ();
					album.Title = stmt.column_text (Columns.ALBUM_TITLE);
					album.Artist = stmt.column_text (Columns.ARTIST_NAME);

					results_model.append (track.Uri, null, 0, track.MimeType, track.Title, track.Artist);

					// FIXME not sure how to deal with albums from an activation point of view.
					if (albums.add (album))
						results_model.append ("uri://need-to-solve-this", null, 1, 
									  		  "need-to/solve-this", album.Title, album.Artist);

					break;
				default:
					printerr ("Error: %d, %s\n", rc, db.errmsg ());
					break;
				}
			} while (rc == Sqlite.ROW);
		}
	}
}